/**
 * 
 */
package arc_eager;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.util.Date;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

import libsvm.svm;
import libsvm.svm_model;
import libsvm.svm_node;

import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;

import svm_utils.svm_predict;
import svm_utils.svm_train;

/**
 * @author Miguel Ballesteros
 *
 */
public class Parser {
	
	public static boolean training=true;
	public static boolean labelled=true;
	private SvmModel svmM;
	private BufferedWriter bw;
	public Parser(){
		svmM=new SvmModel();
		try {
			bw=new BufferedWriter(new FileWriter("output.txt"));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void training(String trainingSet){
		Date d=new Date();
		long tiempoInicial=d.getTime();
		System.out.println("Loading treebank.");
		
		try {
			BufferedReader br=new BufferedReader(new FileReader(trainingSet));
			ArrayList<ArrayList<String[]>> corpus =new ArrayList<ArrayList<String[]>>();
			ArrayList<String[]> cSentence =new ArrayList<String[]>();
			
			while(br.ready()){
				String line=br.readLine();
				if (line.isEmpty()){
					corpus.add(cSentence);
					cSentence=new ArrayList<String[]>();
				}
				else {
					String[] split=line.split("\t");
					cSentence.add(split);
				}
			}
			br.close();
			Oracle oracle=new Oracle();
			
			String[] root=new String[10];
			root[0]="0";
			for(int i=1;i<10;i++) root[i]="ROOT"; //root at the end of the sentence, following Ballesteros and Nivre (CL; 2013)
			root[6]="-1";
			
			for (int i=0;i<corpus.size();i++){
				ArrayList<String[]> buffer=new ArrayList<String[]>();
				ArrayList<String[]> sentence=corpus.get(i);
				buffer.add(root);
				HashMap<String,String> leftMostChilds=new HashMap<String,String>();
				for (int k=0;k<sentence.size();k++){
					String child=sentence.get(k)[0];
					String head=sentence.get(k)[6];
					if (leftMostChilds.get(head)==null){
						leftMostChilds.put(head,child);
					}
				}
				for (int k=sentence.size()-1;k>=0;k--){
					buffer.add(sentence.get(k));
				}
				ArrayList<String[]> stack=new ArrayList<String[]>();
				HashMap<String,String> arcs=new HashMap<String,String>();
				HashMap<String,String> labels=new HashMap<String,String>();

				ArrayList<String> listOfTransitions=new ArrayList<String>();
				while(!buffer.isEmpty()){
					Transition transition=oracle.getTransition(stack, buffer,leftMostChilds,arcs);
					
					svmM.addLine(true, transition.type, transition.label, stack, buffer,labels,listOfTransitions);
					
					listOfTransitions.add(transition.type+"");
					if (transition.type==0){//SHIFT or RIGHT ARC
						stack.add(buffer.get(buffer.size()-1));
						buffer.remove(buffer.size()-1);
						
					}
					else if (transition.type==1){ //REDUCE or LEFT ARC
						stack.remove(stack.size()-1);
					}
					else if (transition.type==2){ //LEFT-ARC
						arcs.put(stack.get(stack.size()-1)[0], buffer.get(buffer.size()-1)[0]);
						labels.put(stack.get(stack.size()-1)[0], transition.label);
						stack.remove(stack.size()-1);
					}
					else if (transition.type==3){//RIGHT-ARC
						arcs.put(buffer.get(buffer.size()-1)[0], stack.get(stack.size()-1)[0]);
						labels.put(buffer.get(buffer.size()-1)[0], transition.label);
						stack.add(buffer.get(buffer.size()-1));
						buffer.remove(buffer.size()-1);
					}
				}
			}
			svmM.closeBuffers(true);
			String[] args =new String[28];
			args[0]="-s";args[1]="0";args[2]="-t";args[3]="1";args[4]="-d";
			args[5]="2";args[6]="-g";args[7]="0.2";args[8]="-r";args[9]="0.0";args[10]="-n";
			args[11]="0.5";args[12]="-n";args[13]="0.5";args[14]="-m";args[15]="100";args[16]="-c";
			args[17]="1.0";args[18]="-e";args[19]="1.0";args[20]="-p";args[21]="0.1";args[22]="-h";args[23]="1";
			args[24]="-b";args[25]="0";args[26]="-q";
			args[27]="svm_train.svm";
			if (training) {
				System.out.println("Training Libsvm model (may take a while)");
				svm_train.main(args);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	public void parsing(String testSet){
		System.out.println("Loading test-set.");
		try {
			BufferedReader br=new BufferedReader(new FileReader(testSet));
			ArrayList<ArrayList<String[]>> corpus =new ArrayList<ArrayList<String[]>>();
			ArrayList<String[]> cSentence =new ArrayList<String[]>();
			while(br.ready()){
				String line=br.readLine();
				if (line.isEmpty()){
					corpus.add(cSentence);
					cSentence=new ArrayList<String[]>();
				}
				else {
					String[] split=line.split("\t");
					cSentence.add(split);
				}
			}
			br.close();
			String[] root=new String[10];
			root[0]="0";
			for(int i=1;i<10;i++) root[i]="ROOT"; //root at the end of the sentence, following Ballesteros and Nivre (CL; 2013)
			root[6]="-1";
			
			
			//svm.svm_predict(model, x);
			
			System.out.println("Parsing...");
			svm_model model=svm.svm_load_model("svm_train.svm.model");
			System.out.println("Model loaded.");
			
			for (int i=0;i<corpus.size();i++){
				ArrayList<String[]> buffer=new ArrayList<String[]>();
				ArrayList<String[]> sentence=corpus.get(i);
				buffer.add(root);
				for (int k=sentence.size()-1;k>=0;k--){
					buffer.add(sentence.get(k));
				}
				ArrayList<String[]> stack=new ArrayList<String[]>();
				HashMap<String,String> arcs=new HashMap<String,String>();
				HashMap<String,String> labels=new HashMap<String,String>();
				ArrayList<String> listOfTransitions=new ArrayList<String>();			
				while(!buffer.isEmpty()){
					Transition transition=new Transition();
					
					if (stack.isEmpty()){
						transition.type=0;
					}
					else {
						svm_node[] nodes=svmM.getNodes(false, -1, "?", stack, buffer,labels,listOfTransitions);
						double out=svm.svm_predict(model,nodes);
						Double d=new Double(out); int outI=d.intValue();
						String s=svmM.getSvmTransitionSingleInt(outI);
						String[] transSplit=s.split("=");
						transition.type=Integer.parseInt(transSplit[0]);
						transition.label=transSplit[1];
					}					
					listOfTransitions.add(transition.type+"");
					if (transition.type==0){//SHIFT
						stack.add(buffer.get(buffer.size()-1));
						buffer.remove(buffer.size()-1);
					}
					else if (transition.type==1){ //REDUCE
							stack.remove(stack.size()-1);
					}
					else if (transition.type==2){ //LEFT-ARC
						arcs.put(stack.get(stack.size()-1)[0], buffer.get(buffer.size()-1)[0]);
						labels.put(stack.get(stack.size()-1)[0], transition.label);
						stack.remove(stack.size()-1);
					}
					else if (transition.type==3){//RIGHT-ARC
						arcs.put(buffer.get(buffer.size()-1)[0], stack.get(stack.size()-1)[0]);
						labels.put(buffer.get(buffer.size()-1)[0], transition.label);
						stack.add(buffer.get(buffer.size()-1));
						buffer.remove(buffer.size()-1);
					}
					
				}
				for (int x=0;x<sentence.size();x++){
					String[] token=sentence.get(x);
					String head=arcs.get(token[0]);
					String label=labels.get(token[0]);
					if (head==null){
						head="0";
					}
					if (label==null || label.equals("null")) label="_";
					bw.write(token[0]+"\t"+token[1]+"\t"+token[2]+"\t"+token[3]+"\t"+token[4]+"\t"+token[5]+"\t"+head+"\t"+label+"\t"+token[8]+"\t"+token[9]+"\n");
				}
				bw.write("\n");
				System.out.print(i+" ");
				if (i%20==0 && i!=0) System.out.println();
			}	
			System.out.println();
			bw.close();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	
	public static void main(String[] args){
		Parser p=new Parser();
		
		
		Option opt = OptionBuilder.withArgName("training")
                .hasArg(true)
                .isRequired(false)
                .withDescription("training")
                .withLongOpt("training")
                .create("t");
		
		Option optL = OptionBuilder.withArgName("labelled")
                .hasArg(true)
                .isRequired(false)
                .withDescription("labelled")
                .withLongOpt("labelled")
                .create("l");
		
		Option ssOpt = OptionBuilder.withArgName("trainingcorpus")
                .hasArg(true)
                .isRequired(false)
                .withDescription("ssynt treebank")
                .withLongOpt("tssynt")
                .create("c");
        
        Option dsOpt = OptionBuilder.withArgName("testset")
                .hasArg(true)
                .isRequired(false)
                .withDescription("dsynt treebank")
                .withLongOpt("tdsynt")
                .create("i");
        
        Options options = new Options();
		options.addOption(opt);options.addOption(optL);options.addOption(ssOpt);options.addOption(dsOpt);
		CommandLineParser parser = new BasicParser();
		CommandLine line;
		try {
			line = parser.parse( options, args );
			training=false;
			if (line.getOptionValue("t").equals("1")) {
				training=true;
			}
			labelled=false;
			if (line.getOptionValue("l").equals("1")) {
				labelled=true;
			}
			p.training(line.getOptionValue("c"));
			p.parsing(line.getOptionValue("i"));
		} catch (org.apache.commons.cli.ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}


