/**
 * 
 */
package arc_eager;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;

import libsvm.svm_node;

/**
 * @author Miguel Ballesteros
 *
 */
public class SvmModel {

	int next=0;
	int nextTransition=0;
	
	private HashMap<String, String> featureTranslation=new HashMap<String,String>();
	private HashMap<String, Integer> featureTranslationInt=new HashMap<String,Integer>();
	private HashMap<String, String> transTranslation=new HashMap<String,String>();
	private HashMap<String, String> transTranslationOtherway=new HashMap<String,String>();
	BufferedWriter bwTrain;
	BufferedWriter bwTest;
	public SvmModel() {
		try {
			bwTrain = new BufferedWriter(new FileWriter("svm_train.svm"));
			bwTest = new BufferedWriter(new FileWriter("svm_test.svm"));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	public String getSVMTransition(String svmTransition) {
		Set<String> transitions=transTranslation.keySet();
		Iterator<String> dpIt=transitions.iterator();
		while (dpIt.hasNext()) {
			String transition=dpIt.next();
			String svmAux=transTranslation.get(transition);
			if (svmAux.equals(svmTransition)) return transition; 
		}
		return null;
	}
	public void closeBuffers(boolean train){
		try {
			if (train) bwTrain.close();
			else bwTest.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	private void addNewFeature (String feature){
		
		String svmFeat=featureTranslation.get(feature);
		if (svmFeat==null) {
			next++;
			featureTranslation.put(feature,""+next+":1");
			featureTranslationInt.put(feature,next);
		}
		
	}
	private void addNewTransition (String transition){
		
		String svmFeat=transTranslation.get(transition);
		if (svmFeat==null) {
			nextTransition++;
			transTranslation.put(transition,""+nextTransition);
			transTranslationOtherway.put(""+nextTransition,transition);
		}
		
	}
	public String getSVMTransitionSingleFile(String file) {
		BufferedReader br;
		try {
			br = new BufferedReader(new FileReader(file));
			String svmTransition="";
			while(br.ready()){
				svmTransition=br.readLine();
			}
			br.close();
			Double transition=Double.parseDouble(svmTransition);
			Integer transInt=transition.intValue();
			
			return transTranslationOtherway.get(""+transInt);
			
			/*Set<String> transitionSet=transTranslation.keySet();
			Iterator<String> dpIt=transitionSet.iterator();
			while (dpIt.hasNext()) {
				String transOutput=dpIt.next();
				String svmAux=transTranslation.get(transOutput);
				if (svmAux.equals(transInt.toString())) return transOutput; 
			}*/
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
		return null;
	}
	
	public void addLine(boolean train, int transition, String label, ArrayList<String[]> stack, ArrayList<String[]> buffer, HashMap<String, String> labels, ArrayList<String> listOfTransitions) {	
		String line="";
		boolean write=false;
		//if (true) {
		if (train) {
			String tranLabel=transition+"="+label;
			addNewTransition(tranLabel);
			line+=transTranslation.get(tranLabel);
		}
		else {
			try {
				bwTest = new BufferedWriter(new FileWriter("svm_test.svm"));
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			line+="1";
		}
		String[] empty=new String[10];
		for(int i=0;i<10;i++) empty[i]="-1"; //root at the end of the sentence, following Ballesteros and Nivre (CL; 2013)
	
		String[] s0=(!stack.isEmpty())?stack.get(0):empty;
		String[] s1=(stack.size()>1)?stack.get(1):empty;
		String[] s2=(stack.size()>2)?stack.get(2):empty;
		String[] s3=(stack.size()>3)?stack.get(3):empty;
		String[] s4=(stack.size()>4)?stack.get(4):empty;
		
		String[] b0=(!buffer.isEmpty())?buffer.get(buffer.size()-1):empty;
		String[] b1=(buffer.size()>1)?buffer.get(buffer.size()-2):empty;
		String[] b2=(buffer.size()>2)?buffer.get(buffer.size()-3):empty;
		String[] b3=(buffer.size()>3)?buffer.get(buffer.size()-4):empty;
		String[] b4=(buffer.size()>4)?buffer.get(buffer.size()-5):empty;
		
		//history
		Iterator<String> itL=listOfTransitions.iterator();
		for (int i=listOfTransitions.size()-1;i>listOfTransitions.size()-9;i--){
			try{
				String historyTrans=listOfTransitions.get(i);
				this.addNewFeature(historyTrans);line+=" "+featureTranslation.get(historyTrans);
			}
			catch(Exception e){
			}
		}
		//POS S0
		this.addNewFeature("posS0="+s0[4]);line+=" "+featureTranslation.get("posS0="+s0[4]);
		//POS S1
		this.addNewFeature("posS1="+s1[4]);line+=" "+featureTranslation.get("posS1="+s1[4]);
		//POS B0
		this.addNewFeature("posB0="+b0[4]);line+=" "+featureTranslation.get("posB0="+b0[4]);
		//POS B1
		this.addNewFeature("posB1="+b1[4]);line+=" "+featureTranslation.get("posB1="+b1[4]);
		//POS B2
		this.addNewFeature("posB2="+b2[4]);line+=" "+featureTranslation.get("posB2="+b2[4]);
		//POS B3
		this.addNewFeature("posB3="+b3[4]);line+=" "+featureTranslation.get("posB3="+b3[4]);
		//DEPREL S0
		this.addNewFeature("deprelS0="+labels.get(s0[0]));line+=" "+featureTranslation.get("deprelS0="+labels.get(s0[0]));
		//FORM S0
		this.addNewFeature("formS0="+s0[1]);line+=" "+featureTranslation.get("formS0="+s0[1]);
		//FORM B0
		this.addNewFeature("formB0="+b0[1]);line+=" "+featureTranslation.get("formB0="+b0[1]);
		//FEAT B0
		this.addNewFeature("featB0="+b0[5]);line+=" "+featureTranslation.get("featB0="+b0[5]);
		//FEAT B1
		this.addNewFeature("featB1="+b1[5]);line+=" "+featureTranslation.get("featB1="+b1[5]);
		//FEAT S0
		this.addNewFeature("featS0="+s0[5]);line+=" "+featureTranslation.get("featS0="+s0[5]);
		//FEAT S1
		this.addNewFeature("featS1="+s1[5]);line+=" "+featureTranslation.get("featS1="+s1[5]);
		try {
			if (train){	
				bwTrain.write(line+"\n");
			}
			else {
				bwTest.write(line+"\n");
			}
		} catch (IOException e) {
			e.printStackTrace();
		}	
					
	}
	
	
	/*svm_node[] x = new svm_node[m];
	for(int j=0;j<m;j++)
	{
		x[j] = new svm_node();
		x[j].index = atoi(st.nextToken());
		x[j].value = atof(st.nextToken());
	}*/
	
	public svm_node[] getNodes(boolean train, int transition, String label, ArrayList<String[]> stack, ArrayList<String[]> buffer, HashMap<String, String> labels, ArrayList<String> listOfTransitions) {	
		ArrayList<svm_node> listNodes=new ArrayList<svm_node>();
		
		String[] empty=new String[10];
		for(int i=0;i<10;i++) empty[i]="-1"; //root at the end of the sentence, following Ballesteros and Nivre (CL; 2013)
	
		String[] s0=(!stack.isEmpty())?stack.get(0):empty;
		String[] s1=(stack.size()>1)?stack.get(1):empty;
		String[] s2=(stack.size()>2)?stack.get(2):empty;
		String[] s3=(stack.size()>3)?stack.get(3):empty;
		String[] s4=(stack.size()>4)?stack.get(4):empty;
		
		String[] b0=(!buffer.isEmpty())?buffer.get(buffer.size()-1):empty;
		String[] b1=(buffer.size()>1)?buffer.get(buffer.size()-2):empty;
		String[] b2=(buffer.size()>2)?buffer.get(buffer.size()-3):empty;
		String[] b3=(buffer.size()>3)?buffer.get(buffer.size()-4):empty;
		String[] b4=(buffer.size()>4)?buffer.get(buffer.size()-5):empty;
		
		//history
		Iterator<String> itL=listOfTransitions.iterator();
		for (int i=listOfTransitions.size()-1;i>listOfTransitions.size()-9;i--){
			try{
				String historyTrans=listOfTransitions.get(i);
				this.addNewFeature(historyTrans);
				svm_node n=new svm_node();
				n.index=featureTranslationInt.get(historyTrans);
				n.value=1;
				listNodes.add(n);
			}
			catch(Exception e){
			}
		}
		//POS S0
		this.addNewFeature("posS0="+s0[4]);svm_node n=new svm_node();n.value=1;n.index=featureTranslationInt.get("posS0="+s0[4]);listNodes.add(n);
		//POS S1
		this.addNewFeature("posS1="+s1[4]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("posS1="+s1[4]);listNodes.add(n);
		//POS B0
		this.addNewFeature("posB0="+b0[4]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("posB0="+b0[4]);listNodes.add(n);
		//POS B1
		this.addNewFeature("posB1="+b1[4]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("posB1="+b1[4]);listNodes.add(n);
		//POS B2
		this.addNewFeature("posB2="+b2[4]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("posB2="+b2[4]);listNodes.add(n);
		//POS B3
		this.addNewFeature("posB3="+b3[4]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("posB3="+b3[4]);listNodes.add(n);
		//DEPREL S0
		this.addNewFeature("deprelS0="+labels.get(s0[0]));n=new svm_node();n.value=1;n.index=featureTranslationInt.get("deprelS0="+labels.get(s0[0]));listNodes.add(n);
		//FORM S0
		this.addNewFeature("formS0="+s0[1]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("formS0="+s0[1]);listNodes.add(n);
		//FORM B0
		this.addNewFeature("formB0="+b0[1]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("formB0="+b0[1]);listNodes.add(n);
		//FEAT B0
		this.addNewFeature("featB0="+b0[5]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("featB0="+b0[5]);listNodes.add(n);
		//FEAT B1
		this.addNewFeature("featB1="+b1[5]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("featB1="+b1[5]);listNodes.add(n);
		//FEAT S0
		this.addNewFeature("featS0="+s0[5]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("featS0="+s0[5]);listNodes.add(n);
		//FEAT S1
		this.addNewFeature("featS1="+s1[5]);n=new svm_node();n.value=1;n.index=featureTranslationInt.get("featS1="+s1[5]);listNodes.add(n);
		
		svm_node[] toRet=new svm_node[listNodes.size()];
		for(int i=0;i<listNodes.size();i++) toRet[i]=listNodes.get(i);
		return toRet;
		
					
	}
	
	public String getSvmTransitionSingleInt(Integer i) {
		Set<String> transitionSet=transTranslation.keySet();
		Iterator<String> dpIt=transitionSet.iterator();
		while (dpIt.hasNext()) {
			String transOutput=dpIt.next();
			String perceptronAux=transTranslation.get(transOutput);
			if (perceptronAux.equals(i.toString())) return transOutput; 
		}
		return null;
	}
	
}
